@@ -7,10 +7,12 @@
 #include "AppStreamUtils.h"
 
 #include "utils.h"
+#include <AppStreamQt/pool.h>
 #include <AppStreamQt/release.h>
 #include <AppStreamQt/screenshot.h>
 #include <AppStreamQt/spdx.h>
 #include <AppStreamQt/version.h>
+#include <Category/Category.h>
 #include <KLocalizedString>
 #include <QDebug>
 #include <QJsonArray>
@@ -177,10 +179,16 @@ QString AppStreamUtils::contentRatingText(const AppStream::Component &appdata)
 
     static QStringList texts = {
         {},
-        i18n("All Audiences"),
-        i18nc("As specified in OARS, intensity of contents", "Mild Content"),
-        i18nc("As specified in OARS, intensity of contents", "Moderate Content"),
-        i18nc("As specified in OARS, intensity of contents", "Intense Content"),
+        i18nc("Open Age Ratings Service (https://hughsie.github.io/oars) description of content suitable for everyone", "All Audiences"),
+        i18nc("Open Age Ratings Service (https://hughsie.github.io/oars) description of content with relatively benign themes only unsuitable for very young "
+              "children, such as minor cartoon violence or mild profanity",
+              "Mild Content"),
+        i18nc("Open Age Ratings Service (https://hughsie.github.io/oars) description of content with some intense themes, such as somewhat realistic "
+              "violence, references to sexuality, or adult profanity",
+              "Moderate Content"),
+        i18nc("Open Age Ratings Service (https://hughsie.github.io/oars) description of mature content that could be quite objectionable or unsuitable for "
+              "young audiences, such as realistic graphic violence, extreme profanity or nudity, or glorification of drug use",
+              "Intense Content"),
     };
     return texts[intensity];
 #else
@@ -229,3 +237,27 @@ uint AppStreamUtils::contentRatingMinimumAge(const AppStream::Component &appdata
     return 0;
 #endif
 }
+
+static void kRemoveDuplicates(QList<AppStream::Component> &input, AppStream::Bundle::Kind kind)
+{
+    QSet<QString> ret;
+    for (auto it = input.begin(); it != input.end();) {
+        const auto key = kind == AppStream::Bundle::KindUnknown ? it->id() : it->bundle(kind).id();
+        if (!ret.contains(key)) {
+            ret << key;
+            ++it;
+        } else {
+            it = input.erase(it);
+        }
+    }
+}
+
+QList<AppStream::Component> AppStreamUtils::componentsByCategories(AppStream::Pool *pool, Category *cat, AppStream::Bundle::Kind kind)
+{
+    QList<AppStream::Component> ret;
+    for (const auto &categoryName : cat->involvedCategories()) {
+        ret += pool->componentsByCategories({categoryName});
+    }
+    kRemoveDuplicates(ret, kind);
+    return ret;
+}
