@@ -21,6 +21,7 @@
 #include <KConfigGroup>
 #include <KDesktopFile>
 #include <KFormat>
+#include <KIO/ApplicationLauncherJob>
 #include <KLocalizedString>
 
 #include <AppStreamQt/release.h>
@@ -29,6 +30,7 @@
 #include <QDir>
 #include <QDirIterator>
 #include <QFileInfo>
+#include <QFutureWatcher>
 #include <QIcon>
 #include <QNetworkAccessManager>
 #include <QNetworkReply>
@@ -38,6 +40,7 @@
 #include <QTemporaryFile>
 #include <QTimer>
 #include <QUrlQuery>
+#include <QtConcurrentRun>
 
 static QString iconCachePath(const AppStream::Icon &icon)
 {
@@ -45,10 +48,14 @@ static QString iconCachePath(const AppStream::Icon &icon)
     return QStringLiteral("%1/icons/%2").arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation), icon.url().fileName());
 }
 
-const QStringList FlatpakResource::s_objects({QStringLiteral("qrc:/qml/FlatpakAttention.qml"),
-                                              QStringLiteral("qrc:/qml/FlatpakRemoveData.qml"),
-                                              QStringLiteral("qrc:/qml/FlatpakOldBeta.qml")});
+const QStringList FlatpakResource::s_objects({
+    QStringLiteral("qrc:/qml/FlatpakAttention.qml"),
+    QStringLiteral("qrc:/qml/FlatpakRemoveData.qml"),
+    QStringLiteral("qrc:/qml/FlatpakOldBeta.qml"),
+    QStringLiteral("qrc:/qml/FlatpakEolReason.qml"),
+});
 const QStringList FlatpakResource::s_bottomObjects({QStringLiteral("qrc:/qml/PermissionsList.qml")});
+Q_GLOBAL_STATIC(QNetworkAccessManager, manager)
 
 FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakInstallation *installation, FlatpakBackend *parent)
     : AbstractResource(parent)
@@ -64,34 +71,33 @@ FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakI
 
     // Start fetching remote icons during initialization
     const auto icons = m_appdata.icons();
-    if (!icons.isEmpty()) {
-        for (const AppStream::Icon &icon : icons) {
-            if (icon.kind() == AppStream::Icon::KindRemote) {
-                const QString fileName = iconCachePath(icon);
-                if (!QFileInfo::exists(fileName)) {
-                    const QDir cacheDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
-                    // Create $HOME/.cache/discover/icons folder
-                    cacheDir.mkdir(QStringLiteral("icons"));
-                    QNetworkAccessManager *manager = new QNetworkAccessManager(this);
-                    connect(manager, &QNetworkAccessManager::finished, this, [this, icon, fileName, manager](QNetworkReply *reply) {
-                        if (reply->error() == QNetworkReply::NoError) {
-                            QByteArray iconData = reply->readAll();
-                            QFile file(fileName);
-                            if (file.open(QIODevice::WriteOnly)) {
-                                file.write(iconData);
-                            }
-                            file.close();
-                            Q_EMIT iconChanged();
-                        }
-                        manager->deleteLater();
-                    });
-                    manager->get(QNetworkRequest(icon.url()));
+    if (icons.count() == 1 && icons.constFirst().kind() == AppStream::Icon::KindRemote) {
+        const auto icon = icons.constFirst();
+        const QString fileName = iconCachePath(icon);
+        if (!QFileInfo::exists(fileName)) {
+            const QDir cacheDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
+            // Create $HOME/.cache/discover/icons folder
+            cacheDir.mkdir(QStringLiteral("icons"));
+            auto reply = manager->get(QNetworkRequest(icon.url()));
+            connect(reply, &QNetworkReply::finished, this, [this, icon, fileName, reply] {
+                if (reply->error() == QNetworkReply::NoError) {
+                    QByteArray iconData = reply->readAll();
+                    QFile file(fileName);
+                    if (file.open(QIODevice::WriteOnly)) {
+                        file.write(iconData);
+                    } else {
+                        qDebug() << "could not find icon for" << packageName() << reply->url();
+                        QIcon::fromTheme(QStringLiteral("package-x-generic")).pixmap(32, 32).toImage().save(fileName);
+                    }
+                    file.close();
+                    Q_EMIT iconChanged();
+                    reply->deleteLater();
                 }
-            }
+            });
         }
     }
 
-    connect(this, &FlatpakResource::stateChanged, this, &FlatpakResource::hasDataButUninstalledChanged);
+    connect(this, &FlatpakResource::stateChanged, this, &FlatpakResource::hasDataChanged);
 }
 
 AppStream::Component FlatpakResource::appstreamComponent() const
@@ -473,25 +479,21 @@ void FlatpakResource::invokeApplication() const
         desktopFileName = appstreamId();
     }
 
-    const QString desktopFile = installPath() + QLatin1String("/export/share/applications/") + desktopFileName;
-    const QString runservice = QStringLiteral(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/discover/runservice");
-    if (QFile::exists(desktopFile) && QFile::exists(runservice)) {
-        QProcess::startDetached(runservice, {desktopFile});
+    KService::Ptr service = KService::serviceByStorageId(desktopFileName);
+
+    if (!service) {
+        qWarning() << "Failed to find service" << desktopFileName;
         return;
     }
 
-    g_autoptr(GCancellable) cancellable = g_cancellable_new();
-    g_autoptr(GError) localError = nullptr;
+    auto *job = new KIO::ApplicationLauncherJob(service);
+    connect(job, &KJob::finished, this, [this, service](KJob *job) {
+        if (job->error()) {
+            Q_EMIT backend()->passiveMessage(i18n("Failed to start '%1': %2", service->name(), job->errorString()));
+        }
+    });
 
-    if (!flatpak_installation_launch(m_installation,
-                                     flatpakName().toUtf8().constData(),
-                                     arch().toUtf8().constData(),
-                                     branch().toUtf8().constData(),
-                                     nullptr,
-                                     cancellable,
-                                     &localError)) {
-        qWarning() << "Failed to launch " << m_appdata.name() << ": " << localError->message;
-    }
+    job->start();
 }
 
 void FlatpakResource::fetchChangelog()
@@ -728,6 +730,28 @@ QString translateSymbolicName(const QStringView &name)
     return name.toString();
 }
 
+QString FlatpakResource::eolReason()
+{
+    if (!m_eolReason.has_value()) {
+        auto futureWatcher = new QFutureWatcher<FlatpakRemoteRef *>(this);
+        connect(futureWatcher, &QFutureWatcher<FlatpakRemoteRef *>::finished, this, [this, futureWatcher]() {
+            g_autoptr(FlatpakRemoteRef) rref = futureWatcher->result();
+            if (rref) {
+                m_eolReason = QString::fromUtf8(flatpak_remote_ref_get_eol(rref));
+                Q_EMIT eolReasonChanged();
+            }
+            futureWatcher->deleteLater();
+        });
+
+        futureWatcher->setFuture(QtConcurrent::run(qobject_cast<FlatpakBackend *>(backend())->threadPool(),
+                                                   &FlatpakRunnables::findRemoteRef,
+                                                   this,
+                                                   qobject_cast<FlatpakBackend *>(backend())->cancellable()));
+        return {};
+    }
+    return m_eolReason.value_or(QString());
+}
+
 void FlatpakResource::loadPermissions()
 {
     QByteArray metaDataBytes = FlatpakRunnables::fetchMetadata(this, NULL);
@@ -777,6 +801,28 @@ void FlatpakResource::loadPermissions()
         description = i18n("Can integrate and communicate with smart cards");
         m_permissions.append(FlatpakPermission(brief, description, "network-card"));
     }
+    if (sockets.contains("cups")) {
+        brief = i18n("Printer Access");
+        description = i18n("Can integrate and communicate with printers");
+        m_permissions.append(FlatpakPermission(brief, description, "document-print"));
+    }
+    if (sockets.contains("gpg-agent")) {
+        brief = i18n("GPG Agent");
+        description = i18n("Allows access to the GPG cryptography service, generally used for signing and reading signed documents");
+        m_permissions.append(FlatpakPermission(brief, description, "gpg"));
+    }
+
+    const QString features = contextGroup.readEntry("features", QString());
+    if (features.contains("bluetooth")) {
+        brief = i18n("Bluetooth Access");
+        description = i18n("Can integrate and communicate with Bluetooth devices");
+        m_permissions.append(FlatpakPermission(brief, description, "network-bluetooth"));
+    }
+    if (features.contains("devel")) {
+        brief = i18n("Low-Level System Access");
+        description = i18n("Can make low-level system calls (e.g. ptrace)");
+        m_permissions.append(FlatpakPermission(brief, description, "project-development"));
+    }
 
     const QString devices = contextGroup.readEntry("devices", QString());
     if (devices.contains("all")) {
@@ -903,9 +949,9 @@ QString FlatpakResource::dataLocation() const
     return QDir::homePath() + QLatin1String("/.var/") + id;
 }
 
-bool FlatpakResource::hasDataButUninstalled() const
+bool FlatpakResource::hasData() const
 {
-    return m_state == None && !dataLocation().isEmpty() && QDir(dataLocation()).exists();
+    return !dataLocation().isEmpty() && QDir(dataLocation()).exists();
 }
 
 void FlatpakResource::clearUserData()
@@ -919,7 +965,7 @@ void FlatpakResource::clearUserData()
     if (!QDir(location).removeRecursively()) {
         qWarning() << "Failed to remove location" << location;
     }
-    Q_EMIT hasDataButUninstalledChanged();
+    Q_EMIT hasDataChanged();
 }
 
 int FlatpakResource::versionCompare(FlatpakResource *resource) const
