@@ -4,8 +4,7 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef PACKAGEKITBACKEND_H
-#define PACKAGEKITBACKEND_H
+#pragma once
 
 #include "PackageKitResource.h"
 #include <AppStreamQt/pool.h>
@@ -26,6 +25,14 @@ class OdrsReviewsBackend;
 class PKResultsStream;
 class PKResolveTransaction;
 
+/** This is either a package name or an appstream id */
+struct PackageOrAppId {
+    QString id;
+    bool isPackageName;
+};
+PackageOrAppId makePackageId(const QString &id);
+PackageOrAppId makeAppId(const QString &id);
+
 class DISCOVERCOMMON_EXPORT PackageKitBackend : public AbstractResourcesBackend
 {
     Q_OBJECT
@@ -76,7 +83,7 @@ public:
     void fetchUpdates();
     int fetchingUpdatesProgress() const override;
 
-    HelpfulError *explainDysfunction() const override;
+    InlineMessage *explainDysfunction() const override;
 
     void addPackageArch(PackageKit::Transaction::Info info, const QString &packageId, const QString &summary);
     void addPackageNotArch(PackageKit::Transaction::Info info, const QString &packageId, const QString &summary);
@@ -98,8 +105,14 @@ Q_SIGNALS:
 
 private:
     friend class PackageKitResource;
+    template<typename T, typename W>
+    T resourcesByPackageNames(const W &names) const;
+
+    template<typename T, typename W>
+    T resourcesByAppNames(const W &names) const;
+
     template<typename T>
-    T resourcesByPackageNames(const QStringList &names) const;
+    T resourcesByComponents(const QList<AppStream::Component> &names) const;
 
     void runWhenInitialized(const std::function<void()> &f, QObject *stream);
 
@@ -121,7 +134,7 @@ private:
     bool m_appstreamInitialized = false;
 
     struct {
-        QHash<QString, AbstractResource *> packages;
+        QHash<PackageOrAppId, AbstractResource *> packages;
         QHash<QString, QStringList> packageToApp;
         QHash<QString, QVector<AppPackageKitResource *>> extendedBy;
     } m_packages;
@@ -133,5 +146,3 @@ private:
     QThreadPool m_threadPool;
     QPointer<PKResolveTransaction> m_resolveTransaction;
 };
-
-#endif // PACKAGEKITBACKEND_H
