@@ -4,9 +4,9 @@
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
-#ifndef PACKAGEKITRESOURCE_H
-#define PACKAGEKITRESOURCE_H
+#pragma once
 
+#include <KService>
 #include <PackageKit/Details>
 #include <PackageKit/Transaction>
 #include <resources/AbstractResource.h>
@@ -81,12 +81,12 @@ public:
 
     virtual QString changelog() const
     {
-        return {};
+        return m_changelog;
     }
 
     bool extendsItself() const;
 
-    void runService(const QStringList &desktopFilePaths) const;
+    void runService(KService::Ptr service) const;
 
 Q_SIGNALS:
     void dependenciesFound(const QJsonArray &dependencies);
@@ -135,8 +135,7 @@ private:
     QMap<PackageKit::Transaction::Info, Ids> m_packages;
     const QString m_summary;
     const QString m_name;
+    QString m_changelog;
     int m_dependenciesCount = -1;
     static const QStringList m_objects;
 };
-
-#endif // PACKAGEKITRESOURCE_H
