@@ -154,10 +154,6 @@ public:
     {
         return {};
     }
-    QString longDescription() override
-    {
-        return {};
-    }
     QString origin() const override
     {
         return {};
@@ -172,19 +168,32 @@ public:
     }
     QString upgradeText() const override
     {
-        return i18np("1 package to upgrade", "%1 packages to upgrade", withoutDuplicates().count());
+        return i18np("1 package will be upgraded", "%1 packages will be upgraded", withoutDuplicates().count());
     }
-    void fetchChangelog() override
+    QString longDescription() override
     {
         QStringList changes;
         const auto resources = withoutDuplicates();
         for (auto res : resources) {
-            const auto versions = res->upgradeText();
-            const auto idx = versions.indexOf(u'\u009C');
-            changes += QStringLiteral("<li>") + res->packageName() + QStringLiteral(": ") + versions.leftRef(idx) + QStringLiteral("</li>\n");
+            const auto changelog = res->changelog();
+            if (changelog.isEmpty()) {
+                changes += i18n("<h3>%1</h3>Upgrade to new version %2<br/>No release notes provided", res->packageName(), res->availableVersion());
+            } else {
+                changes += i18n("<h3>%1</h3>Upgrade to new version %2<br/>Release notes:<blockquote>%3</blockquote>",
+                                res->packageName(),
+                                res->availableVersion(),
+                                changelog);
+            }
         }
         changes.sort();
-        Q_EMIT changelogFetched(QStringLiteral("<ul>") + changes.join(QString()) + QStringLiteral("</ul>\n"));
+        return changes.join(QString());
+    }
+    void fetchChangelog() override
+    {
+        for (auto res : qAsConst(m_resources)) {
+            res->fetchUpdateDetails();
+        }
+        Q_EMIT changelogFetched({});
     }
 
     QString installedVersion() const override
@@ -229,12 +238,14 @@ public:
         const auto toDisconnect = (m_resources - candidates);
         for (auto res : toDisconnect) {
             disconnect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::refreshResource);
+            disconnect(res, &AbstractResource::changelogFetched, this, &SystemUpgrade::longDescriptionChanged);
         }
 
         const auto newCandidates = (candidates - m_resources);
         m_resources = candidates;
         for (auto res : newCandidates) {
             connect(res, &AbstractResource::sizeChanged, this, &SystemUpgrade::refreshResource);
+            connect(res, &AbstractResource::changelogFetched, this, &SystemUpgrade::longDescriptionChanged);
         }
     }
 
