@@ -113,8 +113,8 @@ bool DiscoverNotifier::notifyAboutUpdates() const
 
     // To configure to a random value, execute:
     // kwriteconfig5 --file PlasmaDiscoverUpdates --group Global --key RequiredNotificationInterval 3600
-    const QDateTime timeSinceLastRefresh = m_settings->lastNotificationTime().addSecs(m_settings->requiredNotificationInterval());
-    if (timeSinceLastRefresh.isValid() && timeSinceLastRefresh > QDateTime::currentDateTimeUtc()) {
+    const QDateTime earliestNextNotificationTime = m_settings->lastNotificationTime().addSecs(m_settings->requiredNotificationInterval());
+    if (earliestNextNotificationTime.isValid() && earliestNextNotificationTime > QDateTime::currentDateTimeUtc()) {
         return false;
     }
 
@@ -180,8 +180,13 @@ static bool isConnectionAdequate(const QNetworkConfiguration &network)
 #elif QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
 static bool isConnectionAdequate()
 {
-    const auto transport = QNetworkInformation::instance()->transportMedium();
-    return transport == QNetworkInformation::TransportMedium::Ethernet || transport == QNetworkInformation::TransportMedium::WiFi;
+    const auto info = QNetworkInformation::instance();
+    if (info->supports(QNetworkInformation::Feature::Metered)) {
+        return !info->isMetered();
+    } else {
+        const auto transport = info->transportMedium();
+        return transport == QNetworkInformation::TransportMedium::Ethernet || transport == QNetworkInformation::TransportMedium::WiFi;
+    }
 }
 #endif
 
